let StateChart = {
    chart: null,

    initialiseData: function() {
        this.fetchData(
            this.receiveData.bind(this),
            this.handleError
        );

        // if you want the chart to auto-refresh uncomment the following line
    	window.setInterval(this.refreshData.bind(this), 5000);
    },

    refreshData: function() {
        $('#loading').show();
        this.fetchData(this.receiveRefreshData.bind(this));
    },

    fetchData: function(successCallback, errorCallback) {
        let body = {
            "query": [
                {"Column2": ">0"}
            ],
            "script.prerequest": "SetStates"
        };

        FileMaker.performRequest('POST', '/layouts/VirtualList/_find', body, successCallback, errorCallback);
    },

    receiveRefreshData: function(data) {
        if(null == this.chart) {
            return this.receiveData(data);
        }

        let chartData = this.parseFileMakerData(data);
        let dataSets = this.parseReceivedData(chartData);
        this.chart.data.datasets.forEach((dataset) => {
            dataset.data = dataSets.datasets[0].data;
        });
        this.chart.update();
        $('#loading').hide();
    },

    receiveData: function(data) {
        let chartData = this.parseFileMakerData(data);
        let config = this.chartConfig(chartData),
            ctx = document.getElementById('StateChart').getContext('2d');
        this.chart = new Chart(ctx, config);
        $('#loading').hide();
    },

    chartConfig: function(data) {
        return {
            type: 'line',
            data: this.parseReceivedData(data),
            options: {
                responsive: true,
                title: {
                    display: false,
                    text: 'Users by state'
                },
                tooltips: {
                    mode: 'index',
                    intersect: false,
                },
                hover: {
                    mode: 'nearest',
                    intersect: true
                },
                scales: {
                    xAxes: [{
                        display: true,
                        scaleLabel: {
                            display: true,
                            labelString: 'State'
                        }
                    }],
                    yAxes: [{
                        display: true,
                        scaleLabel: {
                            display: true,
                            labelString: 'Count'
                        }
                    }]
                },
                bezierCurve: false
            }
        }
    },

    parseReceivedData: function(data) {
        return {
            labels: data.labels,
            datasets: [{
                label: 'Number users',
                backgroundColor: '#ff0000',
                borderColor: '#ff0000',
                data: data.values,
                fill: false,
                lineTension: 0,
            }]
        };
    },

    parseFileMakerData: function(data) {
        let labels = [],
            values = [];
        data.forEach(function(state) {
            labels.push(state.fieldData.Column1);
            values.push(state.fieldData.Column2);
        });

        return {
            labels: labels,
            values: values
        }
    },

    handleError: function(error) {
        console.info(error);
        $('#loading').hide();
    }
};