let FileMaker = {
    retried: false,

    logUserIn: function(username, password) {
        this.fetchToken(username, password, Login.loginSuccess, Login.loginError);
    },

    fetchToken: function(username, password, success, error) {
        $.ajax({
            type: "POST",
            url: Config.baseURI + '/sessions',
            dataType: 'json',
            beforeSend: function (xhr) {
                xhr.setRequestHeader("Content-Type", "application/json");
                xhr.setRequestHeader("Authorization", "Basic " + btoa(username + ":" + password));
            },
            success: function(resp) {
                let token = resp.response.token;
                FileMaker.retried = false;
                FileMaker.saveToken(token);

                success(resp);
            }, error: function(response) {
                error(response);
            }
        });
    },

    performRequest: function(method, urlSuffix, data, success, error) {
        let config = {
            type: method,
            url: Config.baseURI + urlSuffix,
            dataType: 'json',
            beforeSend: function (xhr) {
                xhr.setRequestHeader("Authorization", "Bearer " + FileMaker.retrieveToken());
                xhr.setRequestHeader("Content-Type", "application/json");
            },
            success: function(response) {
                // Depending on exactly which endpoint we're calling we either want the data, or the whole response
                // if there's a data set, send that, otherwise the whole response object.
                let data = 'undefined' === typeof response.response.data ? response.response : response.response.data;

                success(data);
            }, error: function(response) {
                let code = parseInt(response.responseJSON.messages[0].code);

                if(401 === code) {
                    // no records found
                    success([]);
                }

                // if the token has expired or is invalid then in theory 952 will come back
                // but sometimes you get 105 missing layout (go figure), so try a token refresh
                if([105, 952].includes(code) && !FileMaker.retried) {
                    FileMaker.retried = true;
                    FileMaker.clearToken();
                    FileMaker.performRequest(method, urlSuffix, data, success, error);

                    return;
                }

                error(response);
            }
        };

        // We should only send data when it's not a GET request, otherwise we end up with additional parameters
        if(method !== 'GET') {
            config.data = JSON.stringify(data);
        }

        $.ajax(config);
    },

    retrieveToken: function() {
        let token = sessionStorage.getItem('fmToken');
        if(!token) {
            this.fetchToken(Config.params.username, Config.params.password, App.error, App.error);
        }

        return sessionStorage.getItem('fmToken');
    },

    saveToken: function(token) {
        sessionStorage.setItem('fmToken', token);
    },


    clearToken: function() {
        sessionStorage.removeItem('fmToken');
        Login.setPublicInterface();
    }

};
